@@ -87,6 +87,10 @@ spark_config_shell_args <- function(config, master) {
   }))
 }
 
+no_databricks_guid <- function() {
+  mget("DATABRICKS_GUID", envir = .GlobalEnv, ifnotfound = "") == ""
+}
+
 #' @name spark-connections
 #'
 #' @examples
@@ -115,6 +119,12 @@ spark_connect <- function(master,
   # validate method
   method <- match.arg(method)
 
+  # A Databricks GUID indicates that it is running on a Databricks cluster,
+  # so if there is no GUID, then method = "databricks" must refer to Databricks Connect
+  if (method == "databricks" && no_databricks_guid()) {
+    method <- "databricks-connect"
+    master <- "local"
+  }
   hadoop_version <- list(...)$hadoop_version
 
   master_override <- spark_config_value(config, "sparklyr.connect.master", NULL)
@@ -174,9 +184,10 @@ spark_connect <- function(master,
     method <- "gateway"
 
   # spark-shell (local install of spark)
-  if (method == "shell" || method == "qubole") {
+  if (method == "shell" || method == "qubole" || method == "databricks-connect") {
     scon <- shell_connection(master = master,
                              spark_home = spark_home,
+                             method = method,
                              app_name = app_name,
                              version = version,
                              hadoop_version = hadoop_version,
@@ -192,8 +203,8 @@ spark_connect <- function(master,
                                spark_master_is_yarn_cluster(master, config)),
                              extensions = extensions,
                              batch = NULL)
-    if (method == "qubole") {
-      scon$method <- "qubole"
+    if (method != "shell") {
+      scon$method <- method
     }
   } else if (method == "livy") {
     scon <- livy_connection(master = master,
@@ -263,6 +274,10 @@ spark_connect <- function(master,
     }
   }, onexit = TRUE)
 
+  if (method == "databricks-connect") {
+    spark_context(scon) %>% invoke("setLocalProperty", "spark.databricks.service.client.type", "sparklyr")
+  }
+
   # add to our internal list
   spark_connections_add(scon)
 
@@ -334,7 +349,7 @@ spark_log_file <- function(sc) {
 
 # TRUE if the Spark Connection is a local install
 spark_connection_is_local <- function(sc) {
-  spark_master_is_local(sc$master)
+  spark_master_is_local(sc$master) && !identical(sc$method, "databricks-connect")
 }
 
 spark_master_is_local <- function(master) {
