@@ -76,6 +76,7 @@ readTypedObject <- function(con, type) {
           "f" = readFastStringArray(con),
           "n" = NULL,
           "j" = getJobj(con, readString(con)),
+          "J" = rjson::fromJSON(readString(con)),
           stop(paste("Unsupported type for deserialization", type)))
 }
 
@@ -85,6 +86,10 @@ readString <- function(con) {
 
   if (stringLen > 0) {
     raw <- read_bin(con, raw(), stringLen, endian = "big")
+    if (is.element("00", raw)) {
+     warning("Input contains embedded nuls, removing.")
+     raw <- raw[raw != "00"]
+    }
     string <- rawToChar(raw)
   }
 
@@ -128,7 +133,11 @@ readType <- function(con) {
 }
 
 readDate <- function(con) {
-  as.Date(readString(con))
+  date_str <- readString(con)
+  if (date_str == "")
+    NA
+  else
+    as.Date(date_str)
 }
 
 readTime <- function(con, n = 1) {
@@ -136,13 +145,11 @@ readTime <- function(con, n = 1) {
     as.POSIXct(character(0))
   else {
     t <- readDouble(con, n)
-    timeNA <- as.POSIXct(0, origin = "1970-01-01", tz = "UTC")
 
     r <- as.POSIXct(t, origin = "1970-01-01", tz = "UTC")
     if (getOption("sparklyr.collect.datechars", FALSE))
       as.character(r)
     else {
-      r[r == timeNA] <- as.POSIXct(NA)
       r
     }
   }
