@@ -67,21 +67,26 @@ ml_gbt_classifier.spark_connection <- function(x, formula = NULL, max_iter = 20,
         probability_col = .args[["probability_col"]],
         raw_prediction_col = .args[["raw_prediction_col"]]
       )
-  ) %>%
-    invoke("setCheckpointInterval", .args[["checkpoint_interval"]]) %>%
-    invoke("setMaxBins", .args[["max_bins"]]) %>%
-    invoke("setMaxDepth", .args[["max_depth"]]) %>%
-    invoke("setMinInfoGain", .args[["min_info_gain"]]) %>%
-    invoke("setMinInstancesPerNode", .args[["min_instances_per_node"]]) %>%
-    invoke("setCacheNodeIds", .args[["cache_node_ids"]]) %>%
-    invoke("setMaxMemoryInMB", .args[["max_memory_in_mb"]]) %>%
-    invoke("setLossType", .args[["loss_type"]]) %>%
-    invoke("setMaxIter", .args[["max_iter"]]) %>%
-    invoke("setStepSize", .args[["step_size"]]) %>%
-    invoke("setSubsamplingRate", .args[["subsampling_rate"]]) %>%
-    jobj_set_param("setFeatureSubsetStrategy", .args[["feature_subset_strategy"]], "2.3.0", "auto") %>%
-    jobj_set_param("setThresholds", .args[["thresholds"]]) %>%
-    jobj_set_param("setSeed", .args[["seed"]])
+  ) %>% (
+    function(obj) {
+      do.call(invoke,
+              c(obj, "%>%", Filter(function(x) !is.null(x),
+                                   list(
+                                        list("setCheckpointInterval", .args[["checkpoint_interval"]]),
+                                        list("setMaxBins", .args[["max_bins"]]),
+                                        list("setMaxDepth", .args[["max_depth"]]),
+                                        list("setMinInfoGain", .args[["min_info_gain"]]),
+                                        list("setMinInstancesPerNode", .args[["min_instances_per_node"]]),
+                                        list("setCacheNodeIds", .args[["cache_node_ids"]]),
+                                        list("setMaxMemoryInMB", .args[["max_memory_in_mb"]]),
+                                        list("setLossType", .args[["loss_type"]]),
+                                        list("setMaxIter", .args[["max_iter"]]),
+                                        list("setStepSize", .args[["step_size"]]),
+                                        list("setSubsamplingRate", .args[["subsampling_rate"]]),
+                                        jobj_set_param_helper(obj, "setFeatureSubsetStrategy", .args[["feature_subset_strategy"]], "2.3.0", "auto"),
+                                        jobj_set_param_helper(obj, "setThresholds", .args[["thresholds"]]),
+                                        jobj_set_param_helper(obj, "setSeed", .args[["seed"]])))))
+    })
 
   new_ml_gbt_classifier(jobj)
 }
