@@ -18,15 +18,18 @@
 #' @export
 ft_string_indexer <- function(x, input_col = NULL, output_col = NULL,
                               handle_invalid = "error", string_order_type = "frequencyDesc",
-                              dataset = NULL, uid = random_string("string_indexer_"), ...) {
+                              uid = random_string("string_indexer_"), ...) {
+  check_dots_used()
   UseMethod("ft_string_indexer")
 }
 
+ml_string_indexer <- ft_string_indexer
 
 #' @export
 ft_string_indexer.spark_connection <- function(x, input_col = NULL, output_col = NULL,
                                                handle_invalid = "error", string_order_type = "frequencyDesc",
-                                               dataset = NULL, uid = random_string("string_indexer_"), ...) {
+                                               uid = random_string("string_indexer_"), ...) {
+
   .args <- list(
     input_col = input_col,
     output_col = output_col,
@@ -34,26 +37,24 @@ ft_string_indexer.spark_connection <- function(x, input_col = NULL, output_col =
     string_order_type = string_order_type,
     uid = uid
   ) %>%
-    ml_validator_string_indexer()
+    validator_ml_string_indexer()
 
-  estimator <- ml_new_transformer(
+  estimator <- spark_pipeline_stage(
     x, "org.apache.spark.ml.feature.StringIndexer",
     input_col = .args[["input_col"]], output_col = .args[["output_col"]], uid = .args[["uid"]]
   ) %>%
-    maybe_set_param("setHandleInvalid", .args[["handle_invalid"]], "2.1.0", "error") %>%
-    maybe_set_param("setStringOrderType", .args[["string_order_type"]], "2.3.0",  "frequencyDesc") %>%
+    jobj_set_param("setHandleInvalid", .args[["handle_invalid"]], "2.1.0", "error") %>%
+    jobj_set_param("setStringOrderType", .args[["string_order_type"]], "2.3.0",  "frequencyDesc") %>%
     new_ml_string_indexer()
 
-  if (is.null(dataset))
-    estimator
-  else
-    ml_fit(estimator, dataset)
+  estimator
+
 }
 
 #' @export
 ft_string_indexer.ml_pipeline <- function(x, input_col = NULL, output_col = NULL,
                                           handle_invalid = "error", string_order_type = "frequencyDesc",
-                                          dataset = NULL, uid = random_string("string_indexer_"), ...) {
+                                          uid = random_string("string_indexer_"), ...) {
 
   stage <- ft_string_indexer.spark_connection(
     x = spark_connection(x),
@@ -61,7 +62,6 @@ ft_string_indexer.ml_pipeline <- function(x, input_col = NULL, output_col = NULL
     output_col = output_col,
     handle_invalid = handle_invalid,
     string_order_type = string_order_type,
-    dataset = dataset,
     uid = uid,
     ...
   )
@@ -71,20 +71,20 @@ ft_string_indexer.ml_pipeline <- function(x, input_col = NULL, output_col = NULL
 #' @export
 ft_string_indexer.tbl_spark <- function(x, input_col = NULL, output_col = NULL,
                                         handle_invalid = "error", string_order_type = "frequencyDesc",
-                                        dataset = NULL, uid = random_string("string_indexer_"), ...) {
+                                        uid = random_string("string_indexer_"), ...) {
   stage <- ft_string_indexer.spark_connection(
     x = spark_connection(x),
     input_col = input_col,
     output_col = output_col,
     handle_invalid = handle_invalid,
     string_order_type = string_order_type,
-    dataset = dataset,
     uid = uid,
     ...
   )
   # backwards compatibility for params argument
   dots <- rlang::dots_list(...)
   if (rlang::has_name(dots, "params") && rlang::is_env(dots$params)) {
+    warning("`params` has been deprecated and will be removed in a future release.", call. = FALSE)
     transformer <- if (is_ml_transformer(stage))
       stage
     else
@@ -103,14 +103,14 @@ ft_string_indexer.tbl_spark <- function(x, input_col = NULL, output_col = NULL,
 }
 
 new_ml_string_indexer <- function(jobj) {
-  new_ml_estimator(jobj, subclass = "ml_string_indexer")
+  new_ml_estimator(jobj, class = "ml_string_indexer")
 }
 
 new_ml_string_indexer_model <- function(jobj) {
   new_ml_transformer(jobj,
                      labels = invoke(jobj, "labels") %>%
                        as.character(),
-                     subclass = "ml_string_indexer_model")
+                     class = "ml_string_indexer_model")
 }
 
 #' @rdname ft_string_indexer
@@ -119,7 +119,7 @@ new_ml_string_indexer_model <- function(jobj) {
 #' @export
 ml_labels <- function(model) model$labels
 
-ml_validator_string_indexer <- function(.args) {
+validator_ml_string_indexer <- function(.args) {
   .args <- validate_args_transformer(.args)
   .args[["handle_invalid"]] <- cast_choice(
     .args[["handle_invalid"]], c("error", "skip", "keep")
