@@ -209,6 +209,7 @@ class Serializer(tracker: JVMObjectTracker) {
       case "map"       => dos.writeByte('e')
       case "jobj"      => dos.writeByte('j')
       case "strarray"  => dos.writeByte('f')
+      case "json"      => dos.writeByte('J')
       case _ => throw new IllegalArgumentException(s"Invalid type $typeStr")
     }
   }
@@ -274,6 +275,9 @@ class Serializer(tracker: JVMObjectTracker) {
         case v: java.sql.Timestamp =>
           writeType(dos, "time")
           writeTime(dos, v)
+        case v: StructTypeAsJSON =>
+          writeType(dos, "json")
+          writeString(dos, v.json)
         case v: org.apache.spark.sql.catalyst.expressions.GenericRowWithSchema =>
           writeType(dos, "list")
           writeInt(dos, v.length)
@@ -362,15 +366,31 @@ class Serializer(tracker: JVMObjectTracker) {
   }
 
   def writeDate(out: DataOutputStream, value: Date): Unit = {
-    writeString(out, value.toString)
+    writeString(
+      out,
+      if (null == value)
+        ""
+      else
+        value.toString
+    )
   }
 
   def writeTime(out: DataOutputStream, value: Time): Unit = {
-    out.writeDouble(value.getTime.toDouble / 1000.0)
+    out.writeDouble(
+      if (null == value)
+        Double.NaN
+      else
+        value.getTime.toDouble / 1000.0
+    )
   }
 
   def writeTime(out: DataOutputStream, value: Timestamp): Unit = {
-    out.writeDouble((value.getTime / 1000).toDouble + value.getNanos.toDouble / 1e9)
+    out.writeDouble(
+      if (null == value)
+        Double.NaN
+      else
+        (value.getTime / 1000).toDouble + value.getNanos.toDouble / 1e9
+    )
   }
 
   def writeString(out: DataOutputStream, value: String): Unit = {
@@ -434,7 +454,10 @@ class Serializer(tracker: JVMObjectTracker) {
 
     value.foreach(v => writeTime(
       out,
-      timestampToUTC(v.getTime())
+      if (null == v)
+        null
+      else
+        timestampToUTC(v.getTime())
     ))
   }
 
