@@ -14,7 +14,6 @@ import org.apache.spark.{SparkEnv, SparkException}
 import scala.util.Try
 
 object Utils {
-
   /**
    * Utilities for collecting columns / Datasets back to R
    */
@@ -67,6 +66,15 @@ object Utils {
     }}
   }
 
+  def collectImplBooleanArrArr(local: Array[Row], idx: Integer): Array[Array[Int]] = {
+    local.map{row => {
+      val el = row(idx).asInstanceOf[scala.collection.mutable.WrappedArray[_]]
+      el.map(e => {
+        if (e.isInstanceOf[Boolean]) if (el.asInstanceOf[Boolean]) 1 else 0 else scala.Int.MinValue
+      }).toArray
+    }}
+  }
+
   def collectImplInteger(local: Array[Row], idx: Integer) = {
     local.map{row => {
       val el = row(idx)
@@ -74,6 +82,15 @@ object Utils {
     }}
   }
 
+  def collectImplIntegerArrArr(local: Array[Row], idx: Integer): Array[Array[Int]] = {
+    local.map{row => {
+      val el = row(idx).asInstanceOf[scala.collection.mutable.WrappedArray[_]]
+      el.map(e =>
+        if (e.isInstanceOf[Int]) e.asInstanceOf[Int] else scala.Int.MinValue
+      ).toArray
+    }}
+  }
+
   def collectImplDouble(local: Array[Row], idx: Integer) = {
     local.map{row => {
       val el = row(idx)
@@ -81,6 +98,15 @@ object Utils {
     }}
   }
 
+  def collectImplDoubleArrArr(local: Array[Row], idx: Integer): Array[Array[Double]] = {
+    local.map{row => {
+      val el = row(idx).asInstanceOf[scala.collection.mutable.WrappedArray[_]]
+      el.map(e =>
+        if (e.isInstanceOf[Double]) e.asInstanceOf[Double] else scala.Double.NaN
+      ).toArray
+    }}
+  }
+
   def collectImplFloat(local: Array[Row], idx: Integer): Array[Double]  = {
     local.map{row => {
       val el = row(idx)
@@ -88,6 +114,15 @@ object Utils {
     }}
   }
 
+  def collectImplFloatArrArr(local: Array[Row], idx: Integer): Array[Array[Double]] = {
+    local.map{row => {
+      val el = row(idx).asInstanceOf[scala.collection.mutable.WrappedArray[_]]
+      el.map(e =>
+        if (e.isInstanceOf[Float]) e.asInstanceOf[Float].toDouble else scala.Double.NaN
+      ).toArray
+    }}
+  }
+
   def collectImplByte(local: Array[Row], idx: Integer): Array[Int] = {
     local.map{row => {
       val el = row(idx)
@@ -95,6 +130,15 @@ object Utils {
     }}
   }
 
+  def collectImplByteArrArr(local: Array[Row], idx: Integer): Array[Array[Int]] = {
+    local.map{row => {
+      val el = row(idx).asInstanceOf[scala.collection.mutable.WrappedArray[_]]
+      el.map(e =>
+        if (e.isInstanceOf[Byte]) e.asInstanceOf[Byte].toInt else scala.Int.MinValue
+      ).toArray
+    }}
+  }
+
   def collectImplShort(local: Array[Row], idx: Integer): Array[Int] = {
     local.map{row => {
       val el = row(idx)
@@ -102,6 +146,15 @@ object Utils {
     }}
   }
 
+  def collectImplShortArrArr(local: Array[Row], idx: Integer): Array[Array[Int]] = {
+    local.map{row => {
+      val el = row(idx).asInstanceOf[scala.collection.mutable.WrappedArray[_]]
+      el.map(e =>
+        if (e.isInstanceOf[Short]) e.asInstanceOf[Short].toInt else scala.Int.MinValue
+      ).toArray
+    }}
+  }
+
   def collectImplLong(local: Array[Row], idx: Integer) = {
     local.map{row => {
       val el = row(idx)
@@ -109,6 +162,15 @@ object Utils {
     }}
   }
 
+  def collectImplLongArrArr(local: Array[Row], idx: Integer): Array[Array[Double]] = {
+    local.map{row => {
+      val el = row(idx).asInstanceOf[scala.collection.mutable.WrappedArray[_]]
+      el.map(e =>
+        if (e.isInstanceOf[Long]) e.asInstanceOf[Long].toDouble else scala.Double.NaN
+      ).toArray
+    }}
+  }
+
   def collectImplForceString(local: Array[Row], idx: Integer, separator: String) = {
     var text = local.map{row => {
       val el = row(idx)
@@ -118,6 +180,16 @@ object Utils {
     if (text.length() > 0) text + separator else text
   }
 
+  def collectImplForceStringArrArr(local: Array[Row], idx: Integer, separator: String): Array[Array[String]] = {
+    local.map{row => {
+      val locale = row(idx).asInstanceOf[scala.collection.mutable.WrappedArray[_]]
+
+      locale.map{e => {
+        if (e != null) e.toString() else "<NA>"
+      }}.toArray
+    }}
+  }
+
   def collectImplString(local: Array[Row], idx: Integer, separator: String) = {
     var text = local.map{row => {
       val el = row(idx)
@@ -127,6 +199,16 @@ object Utils {
     if (text.length() > 0) text + separator else text
   }
 
+  def collectImplStringArrArr(local: Array[Row], idx: Integer, separator: String): Array[Array[String]] = {
+    local.map{row => {
+      val locale = row(idx).asInstanceOf[scala.collection.mutable.WrappedArray[_]]
+
+      locale.map{e => {
+        if (e.isInstanceOf[String]) e.asInstanceOf[String] else "<NA>"
+      }}.toArray
+    }}
+  }
+
   def collectImplDecimal(local: Array[Row], idx: Integer) = {
     local.map{row => {
       val el = row(idx)
@@ -137,6 +219,19 @@ object Utils {
     }}
   }
 
+  def collectImplDecimalArrArr(local: Array[Row], idx: Integer): Array[Array[Double]] = {
+    local.map{row => {
+      val localel = row(idx).asInstanceOf[scala.collection.mutable.WrappedArray[_]]
+
+      localel.map(el => {
+        if (el.isInstanceOf[java.math.BigDecimal])
+          el.asInstanceOf[java.math.BigDecimal].doubleValue
+        else
+          scala.Double.NaN
+      }).toArray
+    }}
+  }
+
   def collectImplVector(local: Array[Row], idx: Integer) = {
     local.map{row => {
       val el = row(idx)
@@ -148,17 +243,43 @@ object Utils {
     }}
   }
 
-  def collectImplTimestamp(local: Array[Row], idx: Integer) = {
+  def collectImplJSON(local: Array[Row], idx: Integer) = {
     local.map{row => {
       val el = row(idx)
-      if (el.isInstanceOf[java.sql.Timestamp]) el.asInstanceOf[java.sql.Timestamp] else new java.sql.Timestamp(0)
+      el match {
+        case _: String => new StructTypeAsJSON(el.asInstanceOf[String])
+        case _ => collectImplDefault(local, idx)
+      }
+    }}
+  }
+
+  def collectImplTimestamp(local: Array[Row], idx: Integer) = {
+    local.map{row => {
+      Try(row.getAs[java.sql.Timestamp](idx)).getOrElse(null)
+    }}
+  }
+
+  def collectImplTimestampArrArr(local: Array[Row], idx: Integer): Array[Array[java.sql.Timestamp]] = {
+    local.map{row => {
+      val el = row(idx).asInstanceOf[scala.collection.mutable.WrappedArray[_]]
+      el.map(e =>
+        Try(e.asInstanceOf[java.sql.Timestamp]).getOrElse(null)
+      ).toArray
     }}
   }
 
   def collectImplDate(local: Array[Row], idx: Integer) = {
     local.map{row => {
-      val el = row(idx)
-      if (el.isInstanceOf[java.sql.Date]) el.asInstanceOf[java.sql.Date] else new java.sql.Date(0)
+      Try(row.getAs[java.sql.Date](idx)).getOrElse(null)
+    }}
+  }
+
+  def collectImplDateArrArr(local: Array[Row], idx: Integer): Array[Array[java.sql.Date]] = {
+    local.map{row => {
+      val el = row(idx).asInstanceOf[scala.collection.mutable.WrappedArray[_]]
+      el.map(e =>
+        Try(e.asInstanceOf[java.sql.Date]).getOrElse(null)
+      ).toArray
     }}
   }
 
@@ -167,7 +288,6 @@ object Utils {
   }
 
   def collectImpl(local: Array[Row], idx: Integer, colType: String, separator: String) = {
-
     val ReDecimalType = "(DecimalType.*)".r
     val ReVectorType  = "(.*VectorUDT.*)".r
 
@@ -189,6 +309,33 @@ object Utils {
 
       case ReDecimalType(_)       => collectImplDecimal(local, idx)
       case ReVectorType(_)        => collectImplVector(local, idx)
+      case StructTypeAsJSON.DType => collectImplJSON(local, idx)
+
+      case "ArrayType(BooleanType,true)"           => collectImplBooleanArrArr(local, idx)
+      case "ArrayType(IntegerType,true)"           => collectImplIntegerArrArr(local, idx)
+      case "ArrayType(DoubleType,true)"            => collectImplDoubleArrArr(local, idx)
+      case "ArrayType(StringType,true)"            => collectImplStringArrArr(local, idx, separator)
+      case "ArrayType(LongType,true)"              => collectImplLongArrArr(local, idx)
+      case "ArrayType(ByteType,true)"              => collectImplByteArrArr(local, idx)
+      case "ArrayType(FloatType,true)"             => collectImplFloatArrArr(local, idx)
+      case "ArrayType(ShortType,true)"             => collectImplShortArrArr(local, idx)
+      case "ArrayType(DecimalType,true)"           => collectImplDecimalArrArr(local, idx)
+      case "ArrayType(TimestampType,true)"         => collectImplTimestampArrArr(local, idx)
+      case "ArrayType(CalendarIntervalType,true)"  => collectImplForceStringArrArr(local, idx, separator)
+      case "ArrayType(DateType,true)"              => collectImplDateArrArr(local, idx)
+
+      case "ArrayType(BooleanType,false)"          => collectImplBooleanArrArr(local, idx)
+      case "ArrayType(IntegerType,false)"          => collectImplIntegerArrArr(local, idx)
+      case "ArrayType(DoubleType,false)"           => collectImplDoubleArrArr(local, idx)
+      case "ArrayType(StringType,false)"           => collectImplStringArrArr(local, idx, separator)
+      case "ArrayType(LongType,false)"             => collectImplLongArrArr(local, idx)
+      case "ArrayType(ByteType,false)"             => collectImplByteArrArr(local, idx)
+      case "ArrayType(FloatType,false)"            => collectImplFloatArrArr(local, idx)
+      case "ArrayType(ShortType,false)"            => collectImplShortArrArr(local, idx)
+      case "ArrayType(DecimalType,false)"          => collectImplDecimalArrArr(local, idx)
+      case "ArrayType(TimestampType,false)"        => collectImplTimestampArrArr(local, idx)
+      case "ArrayType(CalendarIntervalType,false)" => collectImplForceStringArrArr(local, idx, separator)
+      case "ArrayType(DateType,false)"             => collectImplDateArrArr(local, idx)
 
       case "NullType"             => collectImplForceString(local, idx, separator)
 
@@ -201,8 +348,10 @@ object Utils {
   }
 
   def collect(df: DataFrame, separator: String): Array[_] = {
-    val local : Array[Row] = df.collect()
-    val dtypes = df.dtypes
+    val columns = df.columns
+    val (transformed_df, dtypes) = DFCollectionUtils.prepareDataFrameForCollection(df)
+    val local = transformed_df.collect
+
     collectArray(local, dtypes, separator)
   }
 
@@ -338,11 +487,12 @@ object Utils {
         val value = r(idx)
 
         column match {
-          case "integer"  => if (Try(value.toInt).isSuccess) value.toInt else null.asInstanceOf[Int]
-          case "double"  => if (Try(value.toDouble).isSuccess) value.toDouble else null.asInstanceOf[Double]
-          case "logical" => if (Try(value.toBoolean).isSuccess) value.toBoolean else null.asInstanceOf[Boolean]
-          case "timestamp" => if (Try(new java.sql.Timestamp(value.toLong * 1000)).isSuccess) new java.sql.Timestamp(value.toLong * 1000) else null.asInstanceOf[java.sql.Timestamp]
-          case _ => if (value == "NA") null.asInstanceOf[String] else value
+          case "integer"  => if (Try(value.toInt).isSuccess) value.toInt else null
+          case "double"  => if (Try(value.toDouble).isSuccess) value.toDouble else null
+          case "logical" => if (Try(value.toBoolean).isSuccess) value.toBoolean else null
+          case "timestamp" => if (Try(new java.sql.Timestamp(value.toLong * 1000)).isSuccess) new java.sql.Timestamp(value.toLong * 1000) else null
+          case "date" => if (Try(new java.sql.Date(value.toLong * 86400000)).isSuccess) new java.sql.Date(value.toLong * 86400000) else null
+          case _ => if (value == "NA") null else value
         }
       })
 
@@ -373,11 +523,12 @@ object Utils {
         val value = r(idx)
 
         column match {
-          case "integer"   => if (Try(value.toInt).isSuccess) value.toInt else null.asInstanceOf[Int]
-          case "double"    => if (Try(value.toDouble).isSuccess) value.toDouble else null.asInstanceOf[Double]
-          case "logical"   => if (Try(value.toBoolean).isSuccess) value.toBoolean else null.asInstanceOf[Boolean]
-          case "timestamp" => if (Try(new java.sql.Timestamp(value.toLong * 1000)).isSuccess) new java.sql.Timestamp(value.toLong * 1000) else null.asInstanceOf[java.sql.Timestamp]
-          case _ => if (value == "NA") null.asInstanceOf[String] else value
+          case "integer"   => if (Try(value.toInt).isSuccess) value.toInt else null
+          case "double"    => if (Try(value.toDouble).isSuccess) value.toDouble else null
+          case "logical"   => if (Try(value.toBoolean).isSuccess) value.toBoolean else null
+          case "timestamp" => if (Try(new java.sql.Timestamp(value.toLong * 1000)).isSuccess) new java.sql.Timestamp(value.toLong * 1000) else null
+          case "date" => if (Try(new java.sql.Date(value.toLong * 86400000)).isSuccess) new java.sql.Date(value.toLong * 86400000) else null
+          case _ => if (value == "NA") null else value
         }
       })
 
