@@ -1,7 +1,14 @@
 package sparklyr
 
+import org.apache.spark._
+import org.apache.spark.rdd.RDD
+import org.apache.spark.sql._
+import org.apache.spark.sql.types.StructType
+import scala.collection.JavaConversions._
+import org.apache.spark.sql.Row
+
 class WorkerContext(
-  iterator: Iterator[org.apache.spark.sql.Row],
+  iterator: Iterator[Row],
   lock: AnyRef,
   closure: Array[Byte],
   columns: Array[String],
@@ -11,13 +18,8 @@ class WorkerContext(
   context: Array[Byte],
   timeZoneId: String,
   schema: org.apache.spark.sql.types.StructType,
-  options: Map[String, String]) {
-
-  import org.apache.spark._
-  import org.apache.spark.rdd.RDD
-  import org.apache.spark.sql._
-  import org.apache.spark.sql.types.StructType
-  import scala.collection.JavaConversions._
+  options: Map[String, String],
+  barrier: Map[String, Any]) {
 
   private var result: Array[Row] = Array[Row]()
   private var sourceArray: Option[Array[Row]] = None
@@ -96,7 +98,23 @@ class WorkerContext(
     schema
   }
 
+  def getArrowConvertersImpl() : ArrowConvertersImpl = {
+    new ArrowConvertersImpl()
+  }
+
+  def getArrowConverters() : Any = {
+    ArrowConverters
+  }
+
+  def getSqlUtils() : Any = {
+    SQLUtils
+  }
+
   def getOptions() : Map[String, String] = {
     options
   }
+
+  def getBarrier() : Map[String, Any] = {
+    barrier
+  }
 }
